"""Command to interact with the users endpoint."""

import json
import logging
import requests
import typer
import urllib.parse

from app.console import console
from app.constants import ApiPaths, API_KEY_NAME
from rich.table import Table
from typing_extensions import Annotated

logger = logging.getLogger(__name__)

user_commands = typer.Typer()


def create_table() -> Table:
    """Create a table for displaying user information."""
    table = Table(title='Users')
    table.add_column('Username', justify='left')
    table.add_column('Enabled')
    table.add_column('Roles')
    table.add_column('Permissions')
    return table


def get_roles_from_string(roles: str) -> list[str]:
    """Convert a comma-separated string of roles into a list."""
    return [role.strip() for role in roles.split(',')] if roles else []


def get_permissions_from_string(permissions: str) -> dict[str, bool]:
    """Convert a comma-separated string of permissions into a dictionary."""
    applicable_permissions = {}
    pairs = permissions.split(',') if permissions else []
    for pair in pairs:
        if '=' not in pair or pair.count('=') != 1:
            logger.warning('Invalid permission format: %s. Skipping.', pair)
            continue
        key, value = pair.split('=')
        applicable_permissions[key.strip()] = value.strip().lower() == 'true'
    return applicable_permissions


@user_commands.command(name='display')
def print_users(ctx: typer.Context):
    api_endpoint = ctx.obj['api_endpoint']
    api_key = ctx.obj['api_key']

    if not api_endpoint or not api_key:
        raise typer.Exit('API_ENDPOINT and API_KEY must be set via CLI or .env')

    user_url = f'{api_endpoint}{ApiPaths.USERS}'
    headers = {API_KEY_NAME: api_key}

    # Need to paginate through the users
    page = 1
    size = 50
    users = []
    while True:
        response = requests.get(
            user_url, headers=headers, params={'page': page, 'size': size}
        )
        response.raise_for_status()
        items = response.json()['items']
        users.extend(items)
        if len(items) < size:
            break
        page += 1

    if not users:
        console.print('No users found.')
        raise typer.Exit()

    table = create_table()

    for user in users:
        table.add_row(
            user['username'],
            str(user['enabled']),
            ', '.join(user['roles']),
            json.dumps(user['permissions']),
        )

    console.print(table)


@user_commands.command(name='current')
def print_current(ctx: typer.Context):
    api_endpoint = ctx.obj['api_endpoint']
    api_key = ctx.obj['api_key']

    if not api_endpoint or not api_key:
        raise typer.Exit('API_ENDPOINT and API_KEY must be set via CLI or .env')

    user_url = f'{api_endpoint}{ApiPaths.USERS}current/'
    headers = {API_KEY_NAME: api_key}

    response = requests.get(user_url, headers=headers)
    response.raise_for_status()
    user = response.json()

    if not user:
        console.print('User not found.')
        raise typer.Exit()

    table = create_table()

    table.add_row(
        user['username'],
        str(user['enabled']),
        ', '.join(user['roles']),
        json.dumps(user['permissions']),
    )

    console.print(table)


@user_commands.command(name='create')
def create_user(
    ctx: typer.Context,
    username: str = typer.Argument(..., help='Username of the new user'),
    roles: Annotated[
        str, typer.Option(help='Comma separated roles for the user')
    ] = None,
    permissions: Annotated[
        str, typer.Option(help='Comma separated permissions for the user')
    ] = None,
):
    api_endpoint = ctx.obj['api_endpoint']
    api_key = ctx.obj['api_key']
    if not api_endpoint or not api_key:
        raise typer.Exit('API_ENDPOINT and API_KEY must be set via CLI or .env')

    user_url = f'{api_endpoint}{ApiPaths.USERS}'
    headers = {API_KEY_NAME: api_key}

    applicable_roles = get_roles_from_string(roles)
    applicable_permissions = get_permissions_from_string(permissions)

    payload = {
        'username': username,
        'enabled': True,  # Default to enabled
        'roles': applicable_roles,
        'permissions': applicable_permissions,
    }

    response = requests.post(user_url, headers=headers, json=payload)
    response.raise_for_status()

    console.print(f'User {username} created successfully.')


def change_user_enabled(enabled: bool, username: str, ctx: typer.Context):
    api_endpoint = ctx.obj['api_endpoint']
    api_key = ctx.obj['api_key']
    if not api_endpoint or not api_key:
        raise typer.Exit('API_ENDPOINT and API_KEY must be set via CLI or .env')
    user_url = f'{api_endpoint}{ApiPaths.USERS}'
    headers = {API_KEY_NAME: api_key}
    payload = {'username': username, 'enabled': enabled}
    response = requests.patch(user_url, headers=headers, json=payload)
    response.raise_for_status()
    state = 'enabled' if enabled else 'disabled'
    console.print(f'User {username} {state} successfully.')


@user_commands.command(name='enable')
def enable_user(
    ctx: typer.Context,
    username: str = typer.Argument(..., help='Username of the user to enable'),
):
    change_user_enabled(True, username, ctx)


@user_commands.command(name='disable')
def disable_user(
    ctx: typer.Context,
    username: str = typer.Argument(..., help='Username of the user to disable'),
):
    change_user_enabled(False, username, ctx)


@user_commands.command(name='set-roles')
def set_user_roles(
    ctx: typer.Context,
    username: str = typer.Argument(
        ..., help='Username of the user to update roles'
    ),
    roles: Annotated[
        str, typer.Option(help='Comma separated roles for the user')
    ] = None,
):
    api_endpoint = ctx.obj['api_endpoint']
    api_key = ctx.obj['api_key']
    if not api_endpoint or not api_key:
        raise typer.Exit('API_ENDPOINT and API_KEY must be set via CLI or .env')
    user_url = f'{api_endpoint}{ApiPaths.USERS}'
    headers = {API_KEY_NAME: api_key}

    applicable_roles = get_roles_from_string(roles)

    payload = {'username': username, 'roles': applicable_roles}
    response = requests.patch(user_url, headers=headers, json=payload)
    response.raise_for_status()
    console.print(
        f'Roles for user {username} updated successfully to: '
        f'{", ".join(response.json().get("roles", []))}'
    )


@user_commands.command(name='set-permissions')
def set_user_permissions(
    ctx: typer.Context,
    username: str = typer.Argument(
        ..., help='Username of the user to update permissions'
    ),
    permissions: Annotated[
        str, typer.Option(help='Comma separated permissions for the user')
    ] = None,
):
    api_endpoint = ctx.obj['api_endpoint']
    api_key = ctx.obj['api_key']
    if not api_endpoint or not api_key:
        raise typer.Exit('API_ENDPOINT and API_KEY must be set via CLI or .env')
    user_url = f'{api_endpoint}{ApiPaths.USERS}'
    headers = {API_KEY_NAME: api_key}

    applicable_permissions = get_permissions_from_string(permissions)
    if not applicable_permissions:
        raise typer.Exit('No valid permissions provided.')

    payload = {
        'username': username,
        'permissions': applicable_permissions,
    }
    response = requests.patch(user_url, headers=headers, json=payload)
    response.raise_for_status()
    console.print(
        f'Permissions for user {username} updated successfully to: '
        f'{json.dumps(response.json().get("permissions", {}))}'
    )


@user_commands.command(name='generate-api-key')
def generate_api_key(
    ctx: typer.Context,
    username: Annotated[
        str, typer.Argument(help='Username for which to generate the API key')
    ],
):
    """Generate and return a new API key for the given user."""
    api_endpoint = ctx.obj['api_endpoint']
    api_key = ctx.obj['api_key']
    if not api_endpoint or not api_key:
        raise typer.Exit('API_ENDPOINT and API_KEY must be set via CLI or .env')

    user_url = f'{api_endpoint}{ApiPaths.USERS}'
    headers = {API_KEY_NAME: api_key}

    # Ensure the username is URL-encoded to handle special characters
    username = urllib.parse.quote(username)
    response = requests.patch(
        f'{user_url}generate-api-key/{username}', headers=headers
    )
    response.raise_for_status()

    console.print(
        f'New API key generated successfully: {response.json().get("api_key")}'
    )
